@@ -25,20 +25,21 @@
 // DEALINGS IN THE SOFTWARE.
 
 #![cfg(test)]
+use std::fmt;
+use std::net::SocketAddr;
+use std::time::Duration;
 
-use crate::types::error::{CallError, Error};
-use crate::types::v2::{self, Response, RpcError};
-use crate::types::DeserializeOwned;
+use crate::types::error::CallError;
+use crate::types::{self, ErrorResponse, Response, SubscriptionId};
 use crate::{future::ServerHandle, RpcModule, WsServerBuilder};
 use anyhow::anyhow;
 use futures_util::future::join;
+use jsonrpsee_core::{to_json_raw_value, traits::IdProvider, DeserializeOwned, Error};
 use jsonrpsee_test_utils::helpers::*;
 use jsonrpsee_test_utils::mocks::{Id, TestContext, WebSocketTestClient, WebSocketTestError};
 use jsonrpsee_test_utils::TimeoutFutureExt;
-use jsonrpsee_types::to_json_raw_value;
-use jsonrpsee_types::v2::error::invalid_subscription_err;
+use jsonrpsee_types::error::invalid_subscription_err;
 use serde_json::Value as JsonValue;
-use std::{fmt, net::SocketAddr, time::Duration};
 use tracing_subscriber::{EnvFilter, FmtSubscriber};
 
 fn init_logger() {
@@ -69,9 +70,12 @@ async fn server() -> SocketAddr {
 /// It has the following methods:
 ///     sync methods: `say_hello` and `add`
 ///     async: `say_hello_async` and `add_sync`
-///     other: `invalid_params` (always returns `CallError::InvalidParams`), `call_fail` (always returns
-/// 			`CallError::Failed`), `sleep_for` Returns the address together with handles for server future
-///				and server stop.
+///     other: `invalid_params` (always returns `CallError::InvalidParams`),
+///            `call_fail` (always returns `CallError::Failed`),
+///            `sleep_for`
+///            `subscribe_hello` (starts a subscription that doesn't send anything)
+///
+/// Returns the address together with handle for the server.
 async fn server_with_handles() -> (SocketAddr, ServerHandle) {
 	let server = WsServerBuilder::default().build("127.0.0.1:0").with_default_timeout().await.unwrap().unwrap();
 	let mut module = RpcModule::new(());
@@ -119,7 +123,7 @@ async fn server_with_handles() -> (SocketAddr, ServerHandle) {
 	module
 		.register_subscription("subscribe_hello", "subscribe_hello", "unsubscribe_hello", |_, sink, _| {
 			std::thread::spawn(move || loop {
-				let _ = sink;
+				let _ = &sink;
 				std::thread::sleep(std::time::Duration::from_secs(30));
 			});
 			Ok(())
@@ -607,11 +611,11 @@ async fn unsubscribe_twice_should_indicate_error() {
 
 	let unsub_call = call("unsubscribe_hello", vec![sub_id], Id::Num(2));
 	let unsub_2 = client.send_request_text(unsub_call).await.unwrap();
-	let unsub_2_err: RpcError = serde_json::from_str(&unsub_2).unwrap();
+	let unsub_2_err: ErrorResponse = serde_json::from_str(&unsub_2).unwrap();
 	let sub_id = to_json_raw_value(&sub_id).unwrap();
 
 	let err = Some(to_json_raw_value(&format!("Invalid subscription ID={}", sub_id)).unwrap());
-	assert_eq!(unsub_2_err, RpcError::new(invalid_subscription_err(err.as_deref()), v2::Id::Number(2)));
+	assert_eq!(unsub_2_err, ErrorResponse::new(invalid_subscription_err(err.as_deref()), types::Id::Number(2)));
 }
 
 #[tokio::test]
@@ -620,9 +624,47 @@ async fn unsubscribe_wrong_sub_id_type() {
 	let addr = server().await;
 	let mut client = WebSocketTestClient::new(addr).with_default_timeout().await.unwrap().unwrap();
 
-	let unsub =
-		client.send_request_text(call("unsubscribe_hello", vec!["string_is_not_supported"], Id::Num(0))).await.unwrap();
-	let unsub_2_err: RpcError = serde_json::from_str(&unsub).unwrap();
-	let err = Some(to_json_raw_value(&"Invalid subscription ID type, must be integer").unwrap());
-	assert_eq!(unsub_2_err, RpcError::new(invalid_subscription_err(err.as_deref()), v2::Id::Number(0)));
+	let unsub = client.send_request_text(call("unsubscribe_hello", vec![13.99_f64], Id::Num(0))).await.unwrap();
+	let unsub_2_err: ErrorResponse = serde_json::from_str(&unsub).unwrap();
+	let err = Some(to_json_raw_value(&"Invalid subscription ID type, must be Integer or String").unwrap());
+	assert_eq!(unsub_2_err, ErrorResponse::new(invalid_subscription_err(err.as_deref()), types::Id::Number(0)));
+}
+
+#[tokio::test]
+async fn custom_subscription_id_works() {
+	struct HardcodedSubscriptionId;
+
+	impl IdProvider for HardcodedSubscriptionId {
+		fn next_id(&self) -> SubscriptionId<'static> {
+			"0xdeadbeef".to_string().into()
+		}
+	}
+
+	init_logger();
+	let server = WsServerBuilder::default()
+		.set_id_provider(HardcodedSubscriptionId)
+		.build("127.0.0.1:0")
+		.with_default_timeout()
+		.await
+		.unwrap()
+		.unwrap();
+	let addr = server.local_addr().unwrap();
+	let mut module = RpcModule::new(());
+	module
+		.register_subscription("subscribe_hello", "subscribe_hello", "unsubscribe_hello", |_, sink, _| {
+			std::thread::spawn(move || loop {
+				let _ = &sink;
+				std::thread::sleep(std::time::Duration::from_secs(30));
+			});
+			Ok(())
+		})
+		.unwrap();
+	server.start(module).unwrap();
+
+	let mut client = WebSocketTestClient::new(addr).with_default_timeout().await.unwrap().unwrap();
+
+	let sub = client.send_request_text(call("subscribe_hello", Vec::<()>::new(), Id::Num(0))).await.unwrap();
+	assert_eq!(&sub, r#"{"jsonrpc":"2.0","result":"0xdeadbeef","id":0}"#);
+	let unsub = client.send_request_text(call("unsubscribe_hello", vec!["0xdeadbeef"], Id::Num(1))).await.unwrap();
+	assert_eq!(&unsub, r#"{"jsonrpc":"2.0","result":"Unsubscribed","id":1}"#);
 }
